using decePubClient;
using Microsoft.AspNetCore.Components.WebAssembly.Hosting;
using System.Net.Http.Headers;
using System.Text.Json;
using Blazored.LocalStorage;
using decePubClient.Extensions;
using decePubClient.Models;
using decePubClient.Services;
using Toolbelt.Blazor.Extensions.DependencyInjection;
using Microsoft.AspNetCore.Components.Authorization;
using Microsoft.AspNetCore.Components.Web;
using Microsoft.AspNetCore.Components.WebAssembly.Authentication;
using Microsoft.Extensions.DependencyInjection.Extensions;
using Microsoft.Extensions.Options;
using SocialPub.ClientModels;
using collAnon.Client.Services;

var builder = WebAssemblyHostBuilder.CreateDefault(args);
builder.RootComponents.Add<App>("#app");
builder.RootComponents.Add<HeadOutlet>("head::after");

builder.Services.AddBlazorDownloadFile();
builder.Services.AddApiAuthorization();
builder.Services.AddOptions()
	.AddAuthorizationCore(options =>
	{
		options.AddPolicy(Policies.IsAdmin, ExtensionMethods.IsAdminPolicy());
		options.AddPolicy(Policies.IsUser, ExtensionMethods.IsUserPolicy());
		options.AddPolicy(Policies.IsModerator, ExtensionMethods.IsUserModeratorPolicy());
	})
	.AddTransient<AppStatusService>();
	builder.Services.TryAddEnumerable(
			ServiceDescriptor.Singleton<IPostConfigureOptions<RemoteAuthenticationOptions<ApiAuthorizationProviderOptions>>,
					ApiAuthorizationOptionsConfiguration>());
	builder.Services.AddBlazoredLocalStorage(config =>
	{
		config.JsonSerializerOptions.WriteIndented = false;
		config.JsonSerializerOptions.PropertyNameCaseInsensitive = true;
		config.JsonSerializerOptions.IgnoreReadOnlyProperties = true;
		config.JsonSerializerOptions.ReadCommentHandling = JsonCommentHandling.Skip;
		config.JsonSerializerOptions.PropertyNamingPolicy = JsonNamingPolicy.CamelCase;
	})
	.AddScoped<TokenAuthStateProvider>()
	.AddScoped<MessagesService>()
	.AddScoped<AuthenticationStateProvider>(provider => provider.GetRequiredService<TokenAuthStateProvider>())
	.AddSingleton(typeof(CoalescingStringLocalizer))
	//.AddTransient<IHttpService, HttpService>()
	.AddHeadElementHelper(options => options.DisableClientScriptAutoInjection = true)
	.AddLocalization()
	.AddTransient<IStorage, Storage>()
	.AddLogging(lb => lb.SetMinimumLevel(LogLevel.Debug))
	.AddIndexedDb();

builder.Services.AddHttpClient("default", client =>
{
	client.BaseAddress = new Uri(builder.HostEnvironment.BaseAddress);
	client.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/json"));
});

builder.Services.AddSingleton(serviceProvider =>
{
	var conf = serviceProvider.GetRequiredService<IConfiguration>();
	return conf.GetSection(nameof(AppConfiguration)).Get<AppConfiguration>();
});

var host = builder.Build();
await host.SetDefaultCulture();
await host.RunAsync();